#include "TDImpFit.hh"
TDImpFit* TDImpFit::_thisTDImpFit;


TDImpFit::TDImpFit(TH1F * HistToFit)
:_HistToFit(HistToFit)
{
  _binnumber = _HistToFit->GetNbinsX();
  std::cout<<"binnumber = "<<_binnumber<<std::endl;
  _LowEdge = _HistToFit->GetBinCenter(0);
  _HighEdge = _HistToFit->GetBinCenter(_binnumber);  
  _FDFunc = new TF1("_FDFunc",TDImpFit::FD,-0.007,0.007,2);
  _thisTDImpFit=this;
  _ImpFunc = new TF1("_ImpFunc",TDImpFit::ImpFunc,_LowEdge,-0.007,4);

  _HistToFit->SetDirectory(0);
}

TDImpFit::TDImpFit(TH1F * HistToFit, 
                   double charmfraction,
		   double lambda1,
	           double lambda2,
	           double sigmaD,
		   double epsilon1)//,
		   //double epsilon2)
:_HistToFit(HistToFit),
 _charmfraction(charmfraction),
 _lambdaD(1.13*sigmaD),
 _lambda1(lambda1),
 _lambda2(lambda2),
 _sigmaD(sigmaD),
 _epsilon1(epsilon1),
 //_epsilon2(epsilon2),
 _bottomfraction(0)
{
	PrintParameters();
  _binnumber = _HistToFit->GetNbinsX();
  std::cout<<"binnumber = "<<_binnumber<<std::endl;
  _LowEdge = _HistToFit->GetBinLowEdge(0);
  _HighEdge = _HistToFit->GetBinLowEdge(_binnumber);
  _FDFunc = new TF1("_FDFunc",TDImpFit::FD, -0.007, 0.007,(Int_t)2);
  _FDFunc->SetParameter(0,_charmfraction);
  _FDFunc->SetParameter(1,_sigmaD);
  _thisTDImpFit=this;
  _ImpFunc = new TF1("_ImpFunc",TDImpFit::ImpFunc,_LowEdge,-0.007,4);
  _ImpFunc->SetParameter(0,_lambda1);
  _ImpFunc->SetParameter(1,_lambda2);
  _ImpFunc->SetParameter(2,_epsilon1);
  _ImpFunc->SetParameter(3,(_charmfraction/2.0));
  _ImpFunc->SetParLimits(2,0.0,1.0);
}

void TDImpFit::InitiateVariables(double charmfraction,
		                 double lambda1,
			         double lambda2,
			         double sigmaD,
				 double epsilon1)//,
				 //double epsilon2)
{
  _charmfraction=charmfraction;
  _lambda1=lambda1;
  _lambda2=lambda2;
  _sigmaD=sigmaD;
  _epsilon1=epsilon1;
  //_epsilon2=epsilon2;
  _lambdaD=1.13*_sigmaD;
  _FDFunc->SetParameter(0,_charmfraction);
  _FDFunc->SetParameter(1,_sigmaD);
  _ImpFunc->SetParameter(0,_lambda1);
  _ImpFunc->SetParameter(1,_lambda2);
  _ImpFunc->SetParameter(3,_epsilon1);
  _ImpFunc->SetParameter(4,(_charmfraction/2.0));
  _ImpFunc->SetParLimits(3,0.0,1.0);
}  

TDImpFit::~TDImpFit()
{
  delete _FDFunc;
}
void TDImpFit::PrintParameters()
{
std::cout<<"charmfraction "<<_charmfraction<<std::endl;
std::cout<<"lambdaD "<<_lambdaD<<std::endl;
std::cout<<"lambda1 "<<_lambda1<<std::endl;
std::cout<<"lambda2 "<<_lambda2<<std::endl;
std::cout<<"sigmaD "<<_sigmaD<<std::endl;
std::cout<<"epsilon1 "<<_epsilon1<<std::endl;
std::cout<<"bottom fraction "<<_bottomfraction<<std::endl;
}

void TDImpFit::DoInitialFit()
{
  _CanvMan->NextCanvasSlot();
  gPad->SetLogy();
  _HistToFit->Fit("_FDFunc","R");
  _CanvMan->NextCanvasSlot();
  gPad->SetLogy();
  _HistToFit->DrawCopy();
  _FDFunc->DrawCopy("SAME");
  _charmfraction=_FDFunc->GetParameter(0);
  _sigmaD=_FDFunc->GetParameter(1);
  _lambdaD=1.13*_sigmaD;
}
void TDImpFit::DoFinalFit()
{
  _CanvMan->NextCanvasSlot();
  gPad->SetLogy();
  _ImpFunc->DrawCopy();
  _CanvMan->NextCanvasSlot();
  gPad->SetLogy();
  _HistToFit->Fit("_ImpFunc","R");
  _lambda1=_ImpFunc->GetParameter(0);
  _lambda2=_ImpFunc->GetParameter(1);
  _epsilon1=_ImpFunc->GetParameter(2);
  _bottomfraction=_ImpFunc->GetParameter(3);
}

Double_t TDImpFit::FD(Double_t *x, Double_t *par)
{
  double charmfraction = par[0];
  double d0 = x[0];
  double lambdaD = (1.13*par[1]);
  double sigmaD = par[1];

  double Term1Coeff = (1.0-_alpha0)/(2.0*lambdaD);
  double Term1Exp = exp((-1.0)*fabs(d0)/lambdaD);
  double Term1 = Term1Coeff * Term1Exp;
  
  double Term2Coeff = _alpha0/(sqrt(2.0*PI)*sigmaD);
  double Term2Exp = exp((-1.0)*d0*d0/(2.0*sigmaD*sigmaD));
  double Term2 = Term2Coeff * Term2Exp;
  return (Double_t)(charmfraction*(Term1 + Term2));
}

double TDImpFit::FD2(double d0)
{
  double lambdaD = (1.13*_sigmaD);

  double Term1Coeff = (1.0-_alpha0)/(2.0*lambdaD);
  double Term1Exp = exp((-1.0)*fabs(d0)/lambdaD);
  double Term1 = Term1Coeff * Term1Exp;

  double Term2Coeff = _alpha0/(sqrt(2.0*PI)*_sigmaD);
  double Term2Exp = exp((-1.0)*d0*d0/(2.0*_sigmaD*_sigmaD));
  double Term2 = Term2Coeff * Term2Exp;
  return (Term1 + Term2);
}

double TDImpFit::FB(double d0, double lambda1, double lambda2, double epsilon1)
{
  double Term1Coeff = epsilon1/(2.0*lambda1);
  double Term1Exp = exp((-1.0)*fabs(d0)/lambda1);
  double Term1 = Term1Coeff * Term1Exp;
  double Term2Coeff = (1.0-epsilon1)/(2.0*lambda2);
  double Term2Exp = exp((-1.0)*fabs(d0)/lambda1);
  double Term2 = Term2Coeff * Term2Exp;
  //std::cout<<(Term1 + Term2)<<" ";
  return (Term1 + Term2);
}
double TDImpFit::Convolute(double lambda1, double lambda2, double epsilon1, double d0)
{
  double d0prime=0.0;
  double kernel=0.0;
  double convolution=0.0;
  double range = _HighEdge-_LowEdge;
  double binwidth = range/_binnumber;
  //std::cout<<"conv ";
  int helpme=0;
  for(int bin=0; bin<_binnumber; bin++)
  {
    
    d0prime=(bin*binwidth)+_LowEdge;
    //d0prime=_HistToFit->GetBinCenter(bin);
    kernel=FB(d0prime,lambda1,lambda2,epsilon1)*FD2(d0-d0prime)*binwidth;
    convolution=convolution+kernel;
    //std::cout<<bin<<" "<<_binnumber<<"*";
    helpme++;
    if (helpme>100)
    {
	    std::cout<<"***exiting***"<<std::endl;
	    std::cout<<bin<<" "<<_binnumber<<"*"<<_thisTDImpFit->GetBinnumber()<<&_binnumber<<std::endl;
	    return convolution;
    }
  }
  //std::cout<<convolution<<std::endl;
  if (_binnumber!=40) std::cout<<_binnumber<<"*"<<_thisTDImpFit<<&_binnumber<<std::endl;
  return convolution;
  //return 3000;
}
Double_t TDImpFit::ImpFunc(Double_t *x, Double_t *par)
{
  //double temp=par[3]*(_thisTDImpFit->Convolute(par[0],par[1],par[2],x[0]));
  //double 
	return par[3]*(_thisTDImpFit->Convolute(par[0],par[1],par[2],x[0]))+_thisTDImpFit->FD2(x[0]);
  //return temp;
}

